
// Instructions: 
// 1. put the tool's biggest diameter into td
// 2. hit F5 and observe the "effective tool offset" -- all holders for the same tool must use the same tool offset, but this value will be auto-bumped up of the size of the tool necessitates it. put the shown value (or larger) into "to_proposed" and re-build (F5)
// 3. if you're happy with the first holder, F6 and save
// 4. put the tool's next biggest diameter into td, F5 to check, F6+save to get STL
// 5. repeat step 4 for all holders you need for your given tool
// 
// refinements:
// - if you need more rigidity between the tool ring and mount, adjust tool_coverage
// - if you need more play in tightening, adjust gap
// - if you want a thicker or taller ring, adjust th and ht

th=5; // thickness of the holding plastic (mm)
ht=15; // height (mm)
td=65; // tool diameter (mm)
to_proposed=45; // tool offset: how far is the center of the tool from the side of the mount (mm)
gap=2; // gap for tightening (mm)
hd=4; // hole dia (mm)
tool_coverage=0.15; // value between 0 and 1 -- how much is the tool ring connected to the mount side?




$fn=32;

// both mirror and not-mirror by the given axis (for concise symmetry)
module bimirror(mr) {
    for (m = [[0,0,0],mr]) mirror(m) children();
}

module arc(r,a1,a2,ir=0) {
    // normalize to 0..360 (even for negatives)
    a1n = (a1 % 360 + 360) % 360; 
    a2n = (a2 % 360 + 360) % 360;
    difference() {
        circle(r);
        if (ir != 0) circle(ir); // if inner radius given, subtract it away
            
        // get the a1 to interpolate to, adding a revolution if going the long way
        a1next = a2n>a1n ? a1n + 360 : a1n; 
        
        polygon([
            [0,0],
            [cos(1.00*a2n + 0.00*a1next)*2*r,sin(1.00*a2n + 0.00*a1next)*2*r],
            [cos(0.66*a2n + 0.33*a1next)*2*r,sin(0.66*a2n + 0.33*a1next)*2*r],
            [cos(0.33*a2n + 0.66*a1next)*2*r,sin(0.33*a2n + 0.66*a1next)*2*r],
            [cos(0.00*a2n + 1.00*a1next)*2*r,sin(0.00*a2n + 1.00*a1next)*2*r],
        ]);
    }
}

module tool() { 
    translate([0,0, 0]) cylinder(d=44.9,h=7);
    translate([0,0, 7]) cylinder(d=40.8,h=2);
    translate([0,0, 9]) cylinder(d=44.9,h=10);
    translate([0,0,19]) cylinder(d=44.9,d2=75,h=10);
    translate([0,0,29]) cylinder(d=75,h=17.7);
    translate([0,0,46.7]) cylinder(d=75,d2=67.4,h=7);
    translate([0,0,53.7]) cylinder(d=67.4,h=80);
}

//%tool();


module square2(size,aligns="LL",radius=0) {
    real_size = len(size) ? size : [size,size];
    tr = [
        aligns[0]=="C" ? 0 : aligns[0]=="R" ? (-real_size[0]/2) : (+real_size[0]/2),
        aligns[1]=="C" ? 0 : aligns[1]=="R" ? (-real_size[1]/2) : (+real_size[1]/2),
    ];
    translate(tr) {
        if (radius>0) {
            inner_size = [for (v=real_size) v-min(radius*2,v)];
            offset(r=radius) square(inner_size, center=true);
        } else {
            square(size,center=true);
        }
    } 
}

module tool_holder_profile() {
    translate([-37/2,0]) difference() {
        polygon([
            [0,0],
            [-7.727,2.071],
            [-7.727+-5.780,2.071+5.765],
            [37+7.727+5.5780,2.071+5.765],
            [37+7.727,2.071],
            [37,0]
        ]);

        translate([37/2,-35.25+5.244]) circle(d=70.5, $fn=$fn*4);
    }
}

to = max(to_proposed,td/2+th);
echo("effective tool offset =",to);

offset_hole = td>45;

mount_hole_z = offset_hole ? ht*1.5 : ht*0.5;
real_ht = offset_hole ? ht*2 : ht;

difference(convexity=50) {
    linear_extrude(real_ht,convexity=50) {
        // mount interface side, 2d
        difference() {
            union() {
                // wings where the we bolt it to the mount
                bimirror([1,0]) translate([-37/2,0]) rotate(90+90-15) square2([8*2,th],aligns="CL");
                
                // round part that goes flush against the body of the mount
                translate([0,-35.25+5.244]) arc(r=35.25,a1=-63.304/2+90,a2=63.304/2+90,$fn=4*$fn);
            }
            // chop off southern bits (body of the arc)
            translate([0,-th]) square2([80,40],aligns="CR");
        }

        // tool grabber side, 2d
        difference() {
            union() {
                hull() {
                    // circle of the tool holder
                    //translate([0,-to]) offset(th) circle(d=td);
                    translate([0,-to]) arc(r=th+td/2,a1=90-180*tool_coverage,a2=90+180*tool_coverage,ir=td/2);
                    
                    // hull target to join it with the mount interface side
                    square2([37,th],aligns="CR");
                    
                }
                translate([0,-to]) offset(th) circle(d=td);
                // tightening wings
                bimirror([1,0,0]) translate([gap/2,-to-td/2]) square2([th,th+2*hd],aligns="LR");
            }
            // tool cut-out
            translate([0,-to]) circle(d=td,$fn=2*$fn);
            
            // gap for tightening
            translate([0,-to-td/2+1]) square2([gap,th+5],aligns="CR");
        }
    }
    
    // mount tightening screw
    bimirror([1,0]) translate([-37/2-4,0,mount_hole_z]) rotate([0,90,90-15]) cylinder(d=hd,h=th*2+1,center=true);
    
    // tool tightening screw
    translate([0,-to-td/2-th-hd,ht/2]) rotate([0,90]) cylinder(d=hd,h=gap+th*2+1,center=true);
    
    if (offset_hole) {
        bimirror([1,0]) {
            translate([37/2,-th,ht]) cube2([50,td+th*2+20,ht+1],aligns="LRL");
            translate([-37/2,0]) rotate(90+90-15) translate([0,th,ht]) cube2([8+th+5,th*5,ht+1],aligns="LL");
        }
        translate([0,-to,ht]) cube2([td+2*th+1,td,ht+1],aligns="CRL");
    }
    
}

// fake tool
//%translate([0,-to]) color([0,1,1,0.1]) cylinder(d=td,h=30); 